import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

// Repaesentiert eine Zelle eines ASCII-Bildschirms
class Kachel {

	int reihe;
	int spalte;

	// initialisiert die angegebene Kachel mit einem Leerzeichen
	Kachel(int reihe, int spalte) {
		this.reihe = reihe;
		this.spalte = spalte;
	}

	// Zeichnet das uebergebene Zeichen in die entsprechende Kachel
	void zeichne(char zeichen) {
		// Implementierung hier unwichtig
		Terminal.setCursor(this.reihe, this.spalte);
		Terminal.print(zeichen);
	}
}

// Repraesentiert einen rechteckigen ASCII-Bildschirm, der aus einzelnen Kacheln
// besteht
class Bildschirm {

	Kachel[][] kacheln;

	// erzeugt/initialisiert einen Bildschirm der angegebenen Groesse
	Bildschirm(int anzahlReihen, int anzahlSpalten) {
		this.kacheln = new Kachel[anzahlReihen][anzahlSpalten];
		for (int r = 0; r < anzahlReihen; r++) {
			for (int s = 0; s < anzahlSpalten; s++) {
				this.kacheln[r][s] = new Kachel(r, s);
			}
		}
	}

	// liefert die Kacheln der angegebenen Reihe
	// Hinweis: Sie muessen sich nicht um Index-Fehler kuemmern
	Kachel[] getReihe(int reihenIndex) {
		return this.kacheln[reihenIndex];
	}

	// liefert die Kacheln der angegebenen Spalte
	// Hinweis: Sie muessen sich nicht um Index-Fehler kuemmern
	Kachel[] getSpalte(int spaltenIndex) {
		Kachel[] spalte = new Kachel[this.kacheln.length];
		for (int i = 0; i < spalte.length; i++) {
			spalte[i] = this.kacheln[i][spaltenIndex];
		}
		return spalte;
	}
}

public class UE11Aufgabe6 {

	public static void main(String[] args) {
		final int N = 5;
		Bildschirm terminal = new Bildschirm(24, 80);
		Kachel[] kacheln = terminal.getReihe(0);
		UE11Aufgabe6.printZeichen(kacheln, 1, N, '-');
		kacheln = terminal.getReihe(N + 1);
		UE11Aufgabe6.printZeichen(kacheln, 1, N, '-');
		kacheln = terminal.getSpalte(0);
		UE11Aufgabe6.printZeichen(kacheln, 1, N, '|');
		kacheln = terminal.getSpalte(N + 1);
		UE11Aufgabe6.printZeichen(kacheln, 1, N, '|');
	}

	static void printZeichen(Kachel[] kacheln, int von, int bis, char zeichen) {
		for (int i = von; i <= bis; i++) {
			kacheln[i].zeichne(zeichen);
		}
	}

}

// zum Testen

class Terminal {

	private static TerminalFrame frame = new TerminalFrame();

	public static void setCursor(int row, int column) {
		Terminal.frame.setCursor(row, column);
	}

	public static int getCursorRow() {
		return Terminal.frame.getCursorRow();
	}

	public static int getCursorColumn() {
		return Terminal.frame.getCursorColumn();
	}

	public static void clear() {
		Terminal.frame.clear();
	}

	public static void print(Object string) {
		Terminal.frame.print(string.toString());
	}

	public static void println(Object string) {
		Terminal.print(string);
		Terminal.println();
	}

	public static void println() {
		Terminal.frame.println();
	}

}

class TerminalFrame extends JFrame {

	TerminalPanel panel;

	TerminalFrame() {
		super("Terminal");
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					TerminalFrame.this
							.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
					TerminalFrame.this.setLayout(new BorderLayout());
					TerminalFrame.this.panel = new TerminalPanel(
							TerminalFrame.this);
					TerminalFrame.this.add(TerminalFrame.this.panel,
							BorderLayout.CENTER);
					TerminalFrame.this.pack();
					// setResizable(false);

					Dimension screenSize = Toolkit.getDefaultToolkit()
							.getScreenSize();
					TerminalFrame.this.setLocation(
							screenSize.width / 2
									- TerminalFrame.this.getWidth() / 2,
							screenSize.height / 2
									- TerminalFrame.this.getHeight() / 2);

					TerminalFrame.this.setVisible(true);
				}
			});
		} catch (Throwable e) {
		}
	}

	void print(String str) {
		this.panel.print(str);
	}

	void println() {
		this.panel.println();
	}

	void clear() {
		this.panel.clear();
	}

	void setCursor(int row, int column) {
		this.panel.setCursor(row, column);
	}

	public int getCursorRow() {
		return this.panel.getCursorRow();
	}

	public int getCursorColumn() {
		return this.panel.getCursorColumn();
	}

}

class TerminalPanel extends JPanel {

	final static int ROWS = 30;
	final static int COLUMNS = 60;

	TerminalFrame frame;

	int cursorX;
	int cursorY;
	Field[][] field;

	Font panelFont;
	int charHeight;
	int charWidth;

	boolean inputQuiet;
	boolean cursorVisible;

	TerminalPanel(TerminalFrame frame) {
		this.frame = frame;
		this.cursorX = 0;
		this.cursorY = 0;
		this.field = new Field[TerminalPanel.ROWS][TerminalPanel.COLUMNS];
		for (int r = 0; r < TerminalPanel.ROWS; r++) {
			for (int c = 0; c < TerminalPanel.COLUMNS; c++) {
				this.field[r][c] = null;
			}
		}

		this.panelFont = new Font(Font.MONOSPACED, Font.PLAIN, 14);
		this.charHeight = 12;
		this.charWidth = 10;
		this.setPreferredSize(new Dimension(this.charWidth
				* TerminalPanel.COLUMNS, this.charHeight * TerminalPanel.ROWS
				+ 5));

		this.inputQuiet = false;
		this.cursorVisible = false;
	}

	void changeCursorVisibility() {
		this.cursorVisible = !this.cursorVisible;
		this.repaint();
	}

	@Override
	public synchronized void repaint() {
		super.repaint();
	}

	@Override
	public synchronized void paintComponent(final Graphics g) {
		if (EventQueue.isDispatchThread()) {
			this.paintIntern(g);
		} else {
			try {
				SwingUtilities.invokeAndWait(new Runnable() {
					@Override
					public void run() {
						TerminalPanel.this.paintIntern(g);
					}
				});
			} catch (Throwable e) {
			}
		}
	}

	private void paintIntern(Graphics g) {
		super.paintComponent(g);
		g.setColor(Color.white);
		g.fillRect(0, 0, this.charWidth * TerminalPanel.COLUMNS,
				this.charHeight * TerminalPanel.ROWS + 5);
		g.setFont(this.panelFont);
		for (int r = 0; r < TerminalPanel.ROWS; r++) {
			for (int c = 0; c < TerminalPanel.COLUMNS; c++) {
				if (this.field[r][c] != null
						&& this.field[r][c].ch != (char) KeyEvent.VK_ENTER) {
					if (this.field[r][c].isOutput) {
						g.setColor(Color.black);
					} else {
						g.setColor(Color.red);
					}
					g.drawString("" + this.field[r][c], c * this.charWidth,
							(r + 1) * this.charHeight);
				}
			}
		}

		if (this.cursorVisible) {
			// draw Cursor
			g.setColor(Color.blue);
			g.drawString("_", this.cursorX * this.charWidth, (this.cursorY + 1)
					* this.charHeight);
		}
	}

	void clear() {
		for (int r = 0; r < TerminalPanel.ROWS; r++) {
			for (int c = 0; c < TerminalPanel.COLUMNS; c++) {
				this.field[r][c] = null;
			}
		}
	}

	void print(String str) {
		for (int i = 0; i < str.length(); i++) {
			this.field[this.cursorY][this.cursorX] = new Field(str.charAt(i),
					true);
			this.replaceCursor(1);
		}
		this.repaint();
	}

	void print(char ch, boolean isOutput) {
		this.field[this.cursorY][this.cursorX] = new Field(ch, isOutput);
		this.replaceCursor(1);
		this.repaint();
	}

	void back() {
		do {
			if (this.cursorX == 0) {
				this.cursorX = TerminalPanel.COLUMNS - 1;
				if (this.cursorY == 0) {
					this.cursorY = TerminalPanel.ROWS - 1;
				} else {
					this.cursorY--;
				}
			} else {
				this.cursorX--;
			}
		} while (this.field[this.cursorY][this.cursorX] == null);
		this.field[this.cursorY][this.cursorX] = null;
		this.repaint();
	}

	void println() {
		this.field[this.cursorY][this.cursorX] = new Field(
				(char) KeyEvent.VK_ENTER, false);
		this.cursorY = (this.cursorY + 1) % TerminalPanel.ROWS;
		this.cursorX = 0;
		this.repaint();
	}

	void setCursor(int row, int column) {
		this.cursorY = Math.abs(row % TerminalPanel.ROWS);
		this.cursorX = Math.abs(column % TerminalPanel.COLUMNS);
	}

	int getCursorRow() {
		return this.cursorY;
	}

	int getCursorColumn() {
		return this.cursorX;
	}

	void replaceCursor(int dx) {
		this.cursorY = (this.cursorY + (this.cursorX + dx)
				/ TerminalPanel.COLUMNS)
				% TerminalPanel.ROWS;
		this.cursorX = (this.cursorX + dx) % TerminalPanel.COLUMNS;
	}

}

class Field {
	char ch;
	boolean isOutput;

	Field(char ch, boolean isOutput) {
		this.ch = ch;
		this.isOutput = isOutput;
	}

	@Override
	public String toString() {
		return "" + this.ch;
	}
}
